@@ -1,12 +1,13 @@
 from copy import deepcopy
 from math import floor
-from typing import Any, Callable, Optional, Tuple, Union
+from typing import Callable, Optional, Tuple, Union
 
 import torch
 import torch.nn.functional as F
 from pyknos.nflows.nn import nets
 from torch import Tensor, nn, optim, relu
-from torch.nn.utils import clip_grad_norm_
+from torch.distributions import Distribution
+from torch.nn.utils.clip_grad import clip_grad_norm_
 from torch.utils import data
 from torch.utils.data.sampler import SubsetRandomSampler, WeightedRandomSampler
 from tqdm.auto import tqdm
@@ -16,13 +17,15 @@ from sbi.utils.sbiutils import (
     get_simulations_since_round,
     handle_invalid_x,
     standardizing_net,
+    z_score_parser,
 )
+from sbi.utils.torchutils import ensure_theta_batched
 from sbi.utils.user_input_checks import validate_theta_and_x
 
 
 def build_input_layer(
-    batch_theta: Tensor = None,
-    z_score_theta: bool = True,
+    batch_theta: Tensor,
+    z_score_theta: Optional[str] = "independent",
     embedding_net_theta: nn.Module = nn.Identity(),
 ) -> nn.Module:
     r"""Builds input layer for the `RestrictionEstimator` with option to z-score.
@@ -32,14 +35,23 @@ def build_input_layer(
     Args:
         batch_theta: Batch of $\theta$s, used to infer dimensionality and (optional)
             z-scoring.
-        z_score_theta: Whether to z-score $\theta$s passing into the network.
+        z_score_theta: Whether to z-score parameters $\theta$ before passing them into
+            the network, can take one of the following:
+            - `none`, or None: do not z-score.
+            - `independent`: z-score each dimension independently.
+            - `structured`: treat dimensions as related, therefore compute mean and std
+            over the entire batch, instead of per-dimension. Should be used when each
+            sample is, for example, a time series or an image.
         embedding_net_theta: Optional embedding network for $\theta$s.
 
     Returns:
         Input layer with optional embedding net and z-scoring.
     """
-    if z_score_theta:
-        input_layer = nn.Sequential(standardizing_net(batch_theta), embedding_net_theta)
+    z_score_theta_bool, structured_theta = z_score_parser(z_score_theta)
+    if z_score_theta_bool:
+        input_layer = nn.Sequential(
+            standardizing_net(batch_theta, structured_theta), embedding_net_theta
+        )
     else:
         input_layer = embedding_net_theta
 
@@ -51,7 +63,7 @@ def build_classifier(
     hidden_features: int = 100,
     num_blocks: int = 2,
     dropout_probability: float = 0.5,
-    z_score_theta: bool = True,
+    z_score_theta: Optional[str] = "independent",
     embedding_net_theta: nn.Module = nn.Identity(),
 ) -> Callable:
     """
@@ -74,8 +86,13 @@ def build_classifier(
             string.
         dropout_probability: Dropout probability of the classifier if `model` is
             `resnet`.
-        z_score_theta: Whether to z-score the parameters $\theta$ used to train the
-            classifier.
+        z_score_theta: Whether to z-score parameters $\theta$ before passing them into
+            the network, can take one of the following:
+            - `none`, or None: do not z-score.
+            - `independent`: z-score each dimension independently.
+            - `structured`: treat dimensions as related, therefore compute mean and std
+            over the entire batch, instead of per-dimension. Should be used when each
+            sample is, for example, a time series or an image.
         embedding_net_theta: Neural network used to encode the parameters before they
             are passed to the classifier.
 
@@ -84,7 +101,7 @@ def build_classifier(
     """
     if model == "resnet":
 
-        def build_nn(theta):
+        def build_nn(theta) -> nn.Module:
             classifier = nets.ResidualNet(
                 in_features=theta.shape[1],
                 out_features=2,
@@ -101,7 +118,7 @@ def build_classifier(
 
     elif model == "mlp":
 
-        def build_nn(theta):
+        def build_nn(theta) -> nn.Module:
             classifier = nn.Sequential(
                 nn.Linear(theta.shape[1], hidden_features),
                 nn.BatchNorm1d(hidden_features),
@@ -126,13 +143,13 @@ def build_classifier(
 class RestrictionEstimator:
     def __init__(
         self,
-        prior: Any,
+        prior: Distribution,
         model: Union[str, Callable] = "resnet",
         decision_criterion: Union[str, Callable] = "nan",
         hidden_features: int = 100,
         num_blocks: int = 2,
         dropout_probability: float = 0.5,
-        z_score: bool = True,
+        z_score: Optional[str] = "independent",
         embedding_net: nn.Module = nn.Identity(),
     ) -> None:
         r"""
@@ -209,6 +226,7 @@ class RestrictionEstimator:
         if self._valid_or_invalid_criterion == "nan":
             label, _, _ = handle_invalid_x(x)
         else:
+            assert isinstance(self._valid_or_invalid_criterion, Callable)
             label = self._valid_or_invalid_criterion(x)
 
         label = label.long()
@@ -248,7 +266,7 @@ class RestrictionEstimator:
         learning_rate: float = 5e-4,
         validation_fraction: float = 0.1,
         stop_after_epochs: int = 20,
-        max_num_epochs: Optional[int] = None,
+        max_num_epochs: int = 2**31 - 1,
         clip_max_norm: Optional[float] = 5.0,
         loss_importance_weights: Union[bool, float] = False,
         subsample_invalid_sims: Union[float, str] = 1.0,
@@ -280,8 +298,8 @@ class RestrictionEstimator:
                 subsample weights such that the data is balanced.
         """
 
-        theta = torch.cat(self._theta_roundwise)
-        label = torch.cat(self._label_roundwise)
+        theta: Tensor = torch.cat(self._theta_roundwise)
+        label: Tensor = torch.cat(self._label_roundwise)
 
         # Get indices for permutation of the data.
         num_examples = len(theta)
@@ -297,12 +315,14 @@ class RestrictionEstimator:
         # E.g. if there are fewer `valid` datapoints, one might want to sample them
         # more often (i.e. show them to the neural network more often). Such a sampler
         # is implemented below. Also see: https://discuss.pytorch.org/t/29907
-        subsample_weights = torch.ones(num_examples)
+        subsample_weights: Tensor = torch.ones(num_examples)
         if subsample_invalid_sims == "auto":
-            subsample_invalid_sims = float(label.sum()) / float(
-                theta.shape[0] - label.sum()
-            )
-        subsample_weights[~label.bool()] = subsample_invalid_sims
+            subsample_invalid = float(label.sum()) / float(theta.shape[0] - label.sum())
+        else:
+            assert isinstance(subsample_invalid_sims, float)
+            subsample_invalid = subsample_invalid_sims
+
+        subsample_weights[torch.logical_not(label.bool())] = subsample_invalid
 
         subsample_weights = deepcopy(subsample_weights)
         subsample_weights[val_indices] = 0.0
@@ -316,7 +336,9 @@ class RestrictionEstimator:
             batch_size=training_batch_size,
             drop_last=True,
             sampler=WeightedRandomSampler(
-                subsample_weights, int(subsample_weights.sum()), replacement=False
+                subsample_weights.tolist(),
+                int(subsample_weights.sum()),
+                replacement=False,
             ),
         )
         val_loader = data.DataLoader(
@@ -326,7 +348,7 @@ class RestrictionEstimator:
             ),
             shuffle=False,
             drop_last=True,
-            sampler=SubsetRandomSampler(val_indices),
+            sampler=SubsetRandomSampler(val_indices.tolist()),
         )
 
         if self._classifier is None:
@@ -342,21 +364,22 @@ class RestrictionEstimator:
             list(self._classifier.parameters()),
             lr=learning_rate,
         )
-        max_num_epochs = 2 ** 31 - 1 if max_num_epochs is None else max_num_epochs
 
         # Compute the fraction of good simulations in dataset.
         if loss_importance_weights:
             if isinstance(loss_importance_weights, bool):
                 good_sim_fraction = torch.sum(label, dtype=torch.float) / label.shape[0]
-                loss_importance_weights = good_sim_fraction
+                importance_weights = good_sim_fraction
+            else:
+                importance_weights = loss_importance_weights
         else:
-            loss_importance_weights = 0.5
+            importance_weights = 0.5
 
         # Factor of two such that the average learning rate remains the same.
         # Needed because the average of reweigh_factor and 1-reweigh_factor will be 0.5
         # only.
         importance_weights = 2 * torch.tensor(
-            [loss_importance_weights, 1 - loss_importance_weights]
+            [importance_weights, 1 - importance_weights]
         )
 
         criterion = nn.CrossEntropyLoss(importance_weights, reduction="none")
@@ -432,10 +455,16 @@ class RestrictionEstimator:
 
         """
         if classifier is None:
-            classifier = self._classifier
+            assert self._classifier is not None, "Classifier must be trained first."
+            classifier_ = self._classifier
+        else:
+            classifier_ = classifier
+
+        classifier_.zero_grad(set_to_none=True)
+
         return RestrictedPrior(
             self._prior,
-            classifier,
+            classifier_,
             self._first_round_validation_theta,
             self._first_round_validation_label,
             allowed_false_negatives,
@@ -456,6 +485,7 @@ class RestrictionEstimator:
         """
         converged = False
 
+        assert self._classifier is not None, "Classifier must be trained first."
         posterior_nn = self._classifier
 
         # (Re)-start the epoch count with the first epoch or any improvement.
@@ -477,8 +507,8 @@ class RestrictionEstimator:
 class RestrictedPrior:
     def __init__(
         self,
-        prior: Any,
-        classifier: "RestrictionEstimator",
+        prior: Distribution,
+        classifier: nn.Module,
         validation_theta: Tensor,
         validation_label: Tensor,
         allowed_false_negatives: float = 0.0,
@@ -503,6 +533,7 @@ class RestrictedPrior:
         self._validation_label = validation_label
         self._classifier_thr = None
         self._reweigh_factor = None
+        self.acceptance_rate = None
 
         self.tune_rejection_threshold(allowed_false_negatives)
 
@@ -524,6 +555,7 @@ class RestrictedPrior:
         sample_shape: Shape = torch.Size(),
         show_progress_bars: bool = False,
         max_sampling_batch_size: int = 10_000,
+        save_acceptance_rate: bool = False,
     ) -> Tensor:
         """
         Return samples from the `RestrictedPrior`.
@@ -536,9 +568,11 @@ class RestrictedPrior:
             sample_shape: Shape of the returned samples.
             show_progress_bars: Whether or not to show a progressbar during sampling.
             max_sampling_batch_size: Batch size for drawing samples from the posterior.
-            Takes effect only in the second iteration of the loop below, i.e., in case
-            of leakage or `num_samples>max_sampling_batch_size`. Larger batch size
-            speeds up sampling.
+                Takes effect only in the second iteration of the loop below, i.e., in
+                case of leakage or `num_samples>max_sampling_batch_size`. Larger batch
+                size speeds up sampling.
+            save_acceptance_rate: If `True`, the acceptance rate is saved and such that
+                it can potentially be used later in `log_prob()`.
 
         Returns:
             Samples from the `RestrictedPrior`.
@@ -559,7 +593,7 @@ class RestrictedPrior:
         sampling_batch_size = min(num_samples, max_sampling_batch_size)
         while num_remaining > 0:
             # Sample and reject.
-            candidates = self._prior.sample((sampling_batch_size,)).reshape(
+            candidates = self._prior.sample(torch.Size((sampling_batch_size,))).reshape(
                 sampling_batch_size, -1
             )
             are_accepted_by_classifier = self.predict(candidates)
@@ -579,6 +613,9 @@ class RestrictedPrior:
             # fixed batch size.
             sampling_batch_size = max_sampling_batch_size
 
+        if save_acceptance_rate:
+            self.acceptance_rate = torch.as_tensor(acceptance_rate)
+
         pbar.close()
         print(
             f"The classifier rejected {(1.0 - acceptance_rate) * 100:.1f}% of all "
@@ -594,6 +631,90 @@ class RestrictedPrior:
 
         return samples
 
+    def log_prob(
+        self,
+        theta: Tensor,
+        norm_restricted_prior: bool = True,
+        track_gradients: bool = False,
+        prior_acceptance_params: Optional[dict] = None,
+    ) -> Tensor:
+        r"""Returns the log-probability of the restricted prior.
+
+        Args:
+            theta: Parameters $\theta$.
+            norm_restricted_prior: Whether to enforce a normalized restricted prior
+                density. The normalizing factor is calculated via rejection sampling,
+                so if you need speedier but unnormalized log probability estimates set
+                here `norm_restricted_prior=False`. The returned log probability is set
+                to -∞ outside of the restriceted prior support regardless of this
+                setting.
+            track_gradients: Whether the returned tensor supports tracking gradients.
+                This can be helpful for e.g. sensitivity analysis, but increases memory
+                consumption.
+            prior_acceptance_params: A `dict` of keyword arguments to override the
+                default values of `prior_acceptance()`. Possible options are:
+                `num_rejection_samples`, `force_update`, `show_progress_bars`, and
+                `rejection_sampling_batch_size`. These parameters only have an effect
+                if `norm_restricted_prior=True`.
+
+        Returns:
+            `(len(θ),)`-shaped log probability for θ in the support of the restricted
+            prior, -∞ (corresponding to 0 probability) outside.
+        """
+        theta = ensure_theta_batched(torch.as_tensor(theta))
+
+        with torch.set_grad_enabled(track_gradients):
+
+            # Evaluate on device, move back to cpu for comparison with prior.
+            prior_log_prob = self._prior.log_prob(theta)
+            accepted_by_classifer = self.predict(theta)
+
+            masked_log_prob = torch.where(
+                accepted_by_classifer.bool(),
+                prior_log_prob,
+                torch.tensor(float("-inf"), dtype=torch.float32),
+            )
+
+            if prior_acceptance_params is None:
+                prior_acceptance_params = dict()  # use defaults
+            log_factor = (
+                torch.log(self.prior_acceptance(**prior_acceptance_params))
+                if norm_restricted_prior
+                else 0
+            )
+
+            return masked_log_prob - log_factor
+
+    @torch.no_grad()
+    def prior_acceptance(
+        self,
+        num_rejection_samples: int = 10_000,
+        force_update: bool = False,
+        show_progress_bars: bool = False,
+        rejection_sampling_batch_size: int = 10_000,
+    ) -> Tensor:
+        r"""Return the fraction of prior samples accepted by the classifier.
+
+        The factor is estimated from the acceptance probability during rejection
+        sampling from the prior.
+
+        Arguments:
+            num_rejection_samples: Number of samples used to estimate correction factor.
+            show_progress_bars: Whether to show a progress bar during sampling.
+            rejection_sampling_batch_size: Batch size for rejection sampling.
+
+        Returns:
+            Estimated acceptance rate.
+        """
+        if self.acceptance_rate is None or force_update:
+            _ = self.sample(
+                (num_rejection_samples,),
+                show_progress_bars=show_progress_bars,
+                max_sampling_batch_size=rejection_sampling_batch_size,
+                save_acceptance_rate=True,
+            )
+        return self.acceptance_rate  # type:ignore
+
     def predict(self, theta: Tensor) -> Tensor:
         r"""
         Run classifier to predict whether the parameter set is `invalid` or `valid`.
@@ -619,7 +740,7 @@ class RestrictedPrior:
     def tune_rejection_threshold(
         self,
         allowed_false_negatives: Optional[float] = None,
-        reweigh_factor: float = None,
+        reweigh_factor: Optional[float] = None,
         print_fp_rate=False,
         safety_margin: Optional[Union[str, float]] = "frequentist",
     ) -> None:
@@ -649,10 +770,10 @@ class RestrictedPrior:
                 distribution of classifier predictions.
         """
 
-        assert allowed_false_negatives is None or reweigh_factor is None, (
-            "Both the `allowed_false_negatives` and the `reweigh_factor` are set. You "
-            "can only set one of them."
-        )
+        assert (
+            allowed_false_negatives is None or reweigh_factor is None
+        ), """Both the `allowed_false_negatives` and the `reweigh_factor` are set. You
+            can only set one of them."""
         self._reweigh_factor = reweigh_factor
 
         valid_val_theta = self._validation_theta[self._validation_label.bool()]
@@ -674,9 +795,11 @@ class RestrictedPrior:
             else:
                 raise NameError(f"`safety_margin` {safety_margin} not supported.")
         else:
+            assert allowed_false_negatives is not None, "Set allowed_false_negatives!"
             quantile_index = floor(num_valid * allowed_false_negatives)
             self._classifier_thr, _ = torch.kthvalue(clf_probs, quantile_index + 1)
 
+        self._classifier_thr = self._classifier_thr.detach()
         if print_fp_rate:
             self.print_false_positive_rate()
 
