@@ -31,6 +31,7 @@ from sbi.inference import (
     ratio_estimator_based_potential,
     simulate_for_sbi,
 )
+from sbi.inference.posteriors.importance_posterior import ImportanceSamplingPosterior
 from sbi.inference.potentials.base_potential import BasePotential
 from sbi.simulators import diagonal_linear_gaussian, linear_gaussian
 from sbi.utils.get_nn_models import classifier_nn, likelihood_nn, posterior_nn
@@ -53,9 +54,11 @@ from sbi.utils.user_input_checks import (
         (SNLE, "maf", "slice"),
         (SNLE, "nsf", "slice_np"),
         (SNLE, "nsf", "rejection"),
+        (SNLE, "maf", "importance"),
         (SNRE_A, "mlp", "slice_np_vectorized"),
         (SNRE_B, "resnet", "nuts"),
         (SNRE_B, "resnet", "rejection"),
+        (SNRE_B, "resnet", "importance"),
     ],
 )
 @pytest.mark.parametrize("data_device", ("cpu", "cuda:0"))
@@ -113,12 +116,15 @@ def test_training_and_mcmc_on_device(
         kwargs = dict(
             density_estimator=utils.posterior_nn(model=model, num_transforms=2)
         )
+        train_kwargs = dict(force_first_round_loss=True)
     elif method == SNLE:
         kwargs = dict(
             density_estimator=utils.likelihood_nn(model=model, num_transforms=2)
         )
+        train_kwargs = dict()
     elif method in (SNRE_A, SNRE_B):
         kwargs = dict(classifier=utils.classifier_nn(model=model))
+        train_kwargs = dict()
     else:
         raise ValueError()
 
@@ -132,7 +138,7 @@ def test_training_and_mcmc_on_device(
         theta, x = theta.to(data_device), x.to(data_device)
 
         estimator = inferer.append_simulations(theta, x).train(
-            training_batch_size=100, max_num_epochs=max_num_epochs
+            training_batch_size=100, max_num_epochs=max_num_epochs, **train_kwargs
         )
         if method == SNLE:
             potential_fn, theta_transform = likelihood_estimator_based_potential(
@@ -159,6 +165,10 @@ def test_training_and_mcmc_on_device(
             posterior = DirectPosterior(
                 posterior_estimator=estimator, prior=prior
             ).set_default_x(x_o)
+        elif mcmc_method == "importance":
+            posterior = ImportanceSamplingPosterior(
+                potential_fn=potential_fn, proposal=prior
+            )
         else:
             posterior = MCMCPosterior(
                 potential_fn=potential_fn,
@@ -261,15 +271,13 @@ def test_validate_theta_and_x_device(training_device: str, data_device: str) ->
     theta = torch.empty((1, 1)).to(data_device)
     x = torch.empty((1, 1)).to(data_device)
 
-    if training_device != data_device:
-        with pytest.warns(UserWarning):
-            theta, x = validate_theta_and_x(theta, x, training_device=training_device)
-    else:
-        theta, x = validate_theta_and_x(theta, x, training_device=training_device)
+    theta, x = validate_theta_and_x(
+        theta, x, data_device=data_device, training_device=training_device
+    )
 
-    assert str(theta.device) == training_device, (
-        f"Data should have its device converted from '{data_device}' "
-        f"to training_device '{training_device}'."
+    assert str(theta.device) == data_device, (
+        f"Data and parameters must be on the same device but:"
+        f"data device='{data_device}' and training_device='{training_device}'."
     )
 
 
@@ -362,6 +370,7 @@ def test_embedding_nets_integration_training_device(
                 hidden_features=4,
             )
         )
+        train_kwargs = dict()
     elif inference_method == SNLE:
         embedding_net = nn.Linear(in_features=D_theta, out_features=2).to(
             embedding_net_device
@@ -374,6 +383,7 @@ def test_embedding_nets_integration_training_device(
                 num_transforms=2,
             )
         )
+        train_kwargs = dict()
     else:
         embedding_net = nn.Linear(in_features=D_x, out_features=2).to(
             embedding_net_device
@@ -386,6 +396,7 @@ def test_embedding_nets_integration_training_device(
                 num_transforms=2,
             )
         )
+        train_kwargs = dict(force_first_round_loss=True)
 
     with pytest.raises(Exception) if prior_device != training_device else nullcontext():
         inference = inference_method(prior=prior, **nn_kwargs, device=training_device)
@@ -408,10 +419,9 @@ def test_embedding_nets_integration_training_device(
         ) if data_device != training_device else nullcontext():
             density_estimator_append = inference.append_simulations(theta, X)
 
-        with pytest.warns(UserWarning) if (round_idx == 0) and (
-            embedding_net_device != training_device
-        ) else nullcontext():
-            density_estimator_train = density_estimator_append.train(max_num_epochs=2)
+        density_estimator_train = density_estimator_append.train(
+            max_num_epochs=2, **train_kwargs
+        )
 
         posterior = inference.build_posterior(density_estimator_train)
         proposal = posterior.set_default_x(x_o)
@@ -465,7 +475,6 @@ def test_vi_on_gpu(num_dim: int, q: Distribution, vi_method: str, sampling_metho
         num_dim: parameter dimension of the gaussian model
         vi_method: different vi methods
         sampling_method: Different sampling methods
-
     """
 
     device = "cuda:0"
