@@ -12,7 +12,7 @@ from torch.distributions import MultivariateNormal
 
 from sbi import analysis as analysis
 from sbi import utils as utils
-from sbi.analysis import ConditionedMDN, conditonal_potential
+from sbi.analysis import ConditionedMDN, conditional_potential
 from sbi.inference import (
     SNPE_A,
     SNPE_B,
@@ -192,7 +192,9 @@ def test_c2st_snpe_on_linearGaussian_different_dims():
     posterior_estimator = inference.train(
         max_num_epochs=10
     )  # Test whether we can stop and resume.
-    posterior_estimator = inference.train(resume_training=True)
+    posterior_estimator = inference.train(
+        resume_training=True, force_first_round_loss=True
+    )
     posterior = DirectPosterior(
         prior=prior, posterior_estimator=posterior_estimator
     ).set_default_x(x_o)
@@ -372,6 +374,48 @@ def test_api_snpe_c_posterior_correction(sample_with, mcmc_method, prior_str):
     _ = posterior.log_prob(samples)
 
 
+# Testing rejection and mcmc sampling methods.
+@pytest.mark.parametrize(
+    "force_first_round_loss, pass_proposal_to_append",
+    (
+        (True, True),
+        (True, False),
+        (False, True),
+        pytest.param(False, False, marks=pytest.mark.xfail),
+    ),
+)
+def test_api_force_first_round_loss(
+    force_first_round_loss: bool, pass_proposal_to_append: bool
+):
+    """Test that leakage correction applied to sampling works, with both MCMC and
+    rejection.
+
+    """
+
+    num_dim = 2
+    x_o = zeros(1, num_dim)
+
+    # likelihood_mean will be likelihood_shift+theta
+    likelihood_shift = -1.0 * ones(num_dim)
+    likelihood_cov = 0.3 * eye(num_dim)
+    prior = utils.BoxUniform(-2.0 * ones(num_dim), 2.0 * ones(num_dim))
+
+    simulator, prior = prepare_for_sbi(
+        lambda theta: linear_gaussian(theta, likelihood_shift, likelihood_cov), prior
+    )
+    inference = SNPE_C(prior, show_progress_bars=False)
+
+    proposal = prior
+    for _ in range(2):
+        train_proposal = proposal if pass_proposal_to_append else None
+        theta, x = simulate_for_sbi(simulator, proposal, 1000)
+        _ = inference.append_simulations(theta, x, proposal=train_proposal).train(
+            force_first_round_loss=force_first_round_loss, max_num_epochs=2
+        )
+        posterior = inference.build_posterior().set_default_x(x_o)
+        proposal = posterior
+
+
 @pytest.mark.slow
 def test_sample_conditional():
     """
@@ -424,7 +468,11 @@ def test_sample_conditional():
     potential_fn, theta_transform = posterior_estimator_based_potential(
         posterior_estimator, prior=prior, x_o=x_o
     )
-    (conditioned_potential_fn, restricted_tf, restricted_prior,) = conditonal_potential(
+    (
+        conditioned_potential_fn,
+        restricted_tf,
+        restricted_prior,
+    ) = conditional_potential(
         potential_fn=potential_fn,
         theta_transform=theta_transform,
         prior=prior,
